<?php
namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");

use Illuminate\Http\Request;
use App\Jobs\UpdateChargeDiscountJob;
use Config;
use Log;

class ChargeDiscountApplyController extends Controller 
{
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $api;

    public function __construct() {      
        
        $this->rechargeWebhookSecret = env('RECHARGE_API_CLIENT_SECRET',true);

        date_default_timezone_set('US/Eastern');
    }

    /**
     * Apply discount code to the queued charge
     */
    public function updateNextChargeDiscount() {

        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_recharge_webhook($requestData, $received_digest);
        $verified = true;

        if ($verified) {
            $receivedData = json_decode($requestData);

            Log::channel('chargeDiscountApplyLog')->info("Request: ".json_encode($receivedData));
        
            if(!isset($receivedData->order->address_id) || empty($receivedData->order->address_id)) {
                Log::channel('chargeDiscountApplyLog')->info("Error in the webhook response - Missing required 
                parameters in the webhook response or the response is not correct");

            } else if (!isset($receivedData->order->discount_codes) || empty($receivedData->order->discount_codes)) {
                Log::channel('chargeDiscountApplyLog')->info("No discount code is applied in the order, OrderId:".$receivedData->order->id);
            
            } else {
                $response = dispatch(new UpdateChargeDiscountJob($receivedData->order));
            }
        } else {
            Log::channel('chargeDiscountApplyLog')->info("Recharge order create Webhook validation failed");
        }
    }
    
    public function verify_recharge_webhook($data, $received_digest) {

        $calculated_digest = hash('sha256',$this->rechargeWebhookSecret.$data);
        if (hash_equals($received_digest,$calculated_digest)) {
            return true;
        } else {
            return false;
        }
    }   
}
