<?php

namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");
use App\Models\RechargeApi;
use App\Models\Shopify\Product;
use Illuminate\Http\Request;
use Log;

class CheckoutController extends Controller
{

    private $modelObj;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $checkoutReturn;
    private $productObj;
    private $totalPrice = 0;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {

        $this->modelObj   = new RechargeApi();
        $this->productObj = new Product();
    }

    /**
     * Create checkout
     */
    public function CreateCheckout(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $status = "Failed";
            $error = "Bad request";
            $message = "Endpoint parameters are missing";
            Log::channel('checkoutLog')->info("Endpoint parameters are missing");
            return $this->returnData($status, $error, $message, $request->input('callback'));
        }
        
        $parameterLineItemStatus = 0;
        $parameterShippingStatus = 0;
        $checkoutRequest         = array();
        $lineItems               = array();
        $shippingAddress         = array();

        Log::channel('checkoutLog')->info("Request: ".$requestData);
        $receivedData = json_decode($requestData, true);
        
        $otpStatus = !empty($receivedData['checkout']['one_time_purchase']) ? 
        trim($receivedData['checkout']['one_time_purchase']) : false;
        //$otpExtra = 0.9;
       
        if (!empty($receivedData['checkout']['line_items']) && isset($receivedData['checkout']['line_items'])) {

            foreach ($receivedData['checkout']['line_items'] as $checkoutValue) {

                if (empty($checkoutValue['product_id']) || empty($checkoutValue['variant_id']) || 
                empty($checkoutValue['price'])) {

                    $parameterLineItemStatus = 1;
                }
            }

        } else {
            $parameterLineItemStatus = 1;
        }

        if (empty($receivedData['checkout']['line_items']) || $parameterLineItemStatus == 1) {

            $status = "Failed";
            $error = "Bad request";
            $message = "Missing required parameters in the line items";
            Log::channel('checkoutLog')->info("Missing required parameters in the line items");
            return $this->returnData($status, $error, $message, $request->input('callback'));
        }

        if (!empty($receivedData['checkout']['shipping_address'])) {

            if (empty($receivedData['checkout']['shipping_address']['first_name'])
                || empty($receivedData['checkout']['shipping_address']['last_name'])
                || empty($receivedData['checkout']['shipping_address']['address1'])
                || empty($receivedData['checkout']['shipping_address']['city'])
                || empty($receivedData['checkout']['shipping_address']['country'])
                || empty($receivedData['checkout']['shipping_address']['province'])) {

                $parameterShippingStatus = 1;
            }
        }

        if ($parameterShippingStatus == 1) {

            $status = "Failed";
            $error = "Bad request";
            $message = "Missing required parameters in the Shipping Address";
            Log::channel('checkoutLog')->info("Missing required parameters in the Shipping Address");
            return $this->returnData($status, $error, $message, $request->input('callback'));
        }

        $shopifyProductDetails = $this->getShopifyProductDetails();

        if (empty($shopifyProductDetails) || !isset($shopifyProductDetails)) {

            $status = "Failed";
            $error = "Bad response";
            $message = "Failed to get the product details from Shopify";
            Log::channel('checkoutLog')->info("Failed to get the product details from Shopify");
            return $this->returnData($status, $error, $message, $request->input('callback'));
        }

        $productVariantErrorStatus = 0;
        $productSku = '';
        $productTitle = '';

        foreach ($receivedData['checkout']['line_items'] as $itemValue) {

            $requestedVariantId = '';
            $variantDetails = [];
            if (!empty($itemValue['product_id']) && !empty($itemValue['variant_id'])) {

                $requestedVariantId = $itemValue['variant_id'];
                $variantDetails = !empty($shopifyProductDetails[$requestedVariantId]) ? 
                $shopifyProductDetails[$requestedVariantId] : '';

                if (empty($variantDetails) || !isset($variantDetails)) {

                    $productVariantErrorStatus = 1;
                    break;
                }
                $productSku = $variantDetails['sku'];
                $productTitle = $variantDetails['product_title'];
            }
        }

        $productId = '';
        $variantId = '';
        $productQuantity = '';
        $productPrice = '';

        $productId = !empty($receivedData['checkout']['line_items'][0]['product_id']) ? 
        trim($receivedData['checkout']['line_items'][0]['product_id']) : '';
        $variantId = !empty($receivedData['checkout']['line_items'][0]['variant_id']) ? 
        trim($receivedData['checkout']['line_items'][0]['variant_id']) : '';
        $productQuantity = !empty($receivedData['checkout']['line_items'][0]['quantity']) ? 
        trim($receivedData['checkout']['line_items'][0]['quantity']) : 1;
        $productPrice = !empty($receivedData['checkout']['line_items'][0]['price']) ? 
        trim($receivedData['checkout']['line_items'][0]['price']) : '';
        $sku = !empty($receivedData['checkout']['line_items'][0]['sku']) ? 
        trim($receivedData['checkout']['line_items'][0]['sku']) : '';

        if (empty($productId) || empty($variantId) || empty($productPrice)) {

            $status = "Failed";
            $error = "Bad request";
            $message = "Missing required parameters in the line items";
            Log::channel('checkoutLog')->info("Missing required parameters in the line items");
            return $this->returnData($status, $error, $message, $request->input('callback'));
        }

        $lineItemProperties = [];
        //$lineItemProperties['SKU'] = $productSku;
        if ($otpStatus && !empty($otpStatus)) {

            $lineItemProperties['one_time_purchase'] = 'true';
        }
        
        if (isset($receivedData['checkout']['line_items'])) {

            foreach ($receivedData['checkout']['line_items'] as $checkoutValue) {

                $lineItemTemp = array(
                    "price"                                   => $productPrice,
                    "product_id"                              => $productId,
                    "properties"                              => $lineItemProperties,
                    "quantity"                                => $productQuantity,
                    "variant_id"                              => $variantId,
                    "cutoff_day_of_month"                     => !empty($checkoutValue['cutoff_day_of_month'])
                        ? $checkoutValue['cutoff_day_of_month'] : '',
                    "cutoff_day_of_week"                      => !empty($checkoutValue['cutoff_day_of_week'])
                        ? $checkoutValue['cutoff_day_of_week'] : '',
                    "expire_after_specific_number_of_charges" => !empty($checkoutValue['expire_after_specific_number_of_charges'])
                        ? $checkoutValue['expire_after_specific_number_of_charges'] : '',
                    "fulfillment_service"                     => "manual",
                    "grams"                                   => !empty($checkoutValue['grams']) ? $checkoutValue['grams'] : '',
                    "line_price"                              => !empty($checkoutValue['line_price']) ? $checkoutValue['line_price'] : '',
                    "order_day_of_month"                      => !empty($checkoutValue['order_day_of_month'])
                        ? $checkoutValue['order_day_of_month'] : '',
                    "order_day_of_week"                       => !empty($checkoutValue['order_day_of_week'])
                        ? $checkoutValue['order_day_of_week'] : '',
                    "sku"                                     => $productSku,
                    "title"                                   => $productTitle,
                    "variant_title"                           => !empty($checkoutValue['variant_title'])
                        ? $checkoutValue['variant_title'] : '',
                    "vendor"                                  => !empty($checkoutValue['vendor']) ? $checkoutValue['vendor'] : '',
                );

                if (!$otpStatus && empty($otpStatus)) {

                    $lineItemTemp["charge_interval_frequency"] = $checkoutValue['charge_interval_frequency'];
                    $lineItemTemp["order_interval_frequency"]  = $checkoutValue['order_interval_frequency'];
                    $lineItemTemp["order_interval_unit"]       = $checkoutValue['order_interval_unit'];
                }

                foreach ($lineItemTemp as $key => $value) {
                    if (empty($value)) {
                        unset($lineItemTemp[$key]);
                    }
                }

                $lineItems[] = $lineItemTemp;
            }
        }

        if (!empty($receivedData['checkout']['shipping_address'])) {

            $shippingAddress = array(
                "address1"   => !empty($receivedData['checkout']['shipping_address']['address1'])
                    ? $receivedData['checkout']['shipping_address']['address1'] : '',
                "address2"   => !empty($receivedData['checkout']['shipping_address']['address2'])
                    ? $receivedData['checkout']['shipping_address']['address2'] : '',
                "city"       => !empty($receivedData['checkout']['shipping_address']['city'])
                    ? $receivedData['checkout']['shipping_address']['city'] : '',
                "company"    => !empty($receivedData['checkout']['shipping_address']['company'])
                    ? $receivedData['checkout']['shipping_address']['company'] : '',
                "country"    => !empty($receivedData['checkout']['shipping_address']['country'])
                    ? $receivedData['checkout']['shipping_address']['country'] : '',
                "first_name" => !empty($receivedData['checkout']['shipping_address']['first_name'])
                    ? $receivedData['checkout']['shipping_address']['first_name'] : '',
                "last_name"  => !empty($receivedData['checkout']['shipping_address']['last_name'])
                    ? $receivedData['checkout']['shipping_address']['last_name'] : '',
                "phone"      => !empty($receivedData['checkout']['shipping_address']['phone'])
                    ? $receivedData['checkout']['shipping_address']['phone'] : '',
                "province"   => !empty($receivedData['checkout']['shipping_address']['province'])
                    ? $receivedData['checkout']['shipping_address']['province'] : '',
                "zip"        => !empty($receivedData['checkout']['shipping_address']['zip'])
                    ? $receivedData['checkout']['shipping_address']['zip'] : ''

            );

            foreach ($shippingAddress as $key => $value) {
                if (empty($value)) {
                    unset($shippingAddress[$key]);
                }
            }
        }

        $checkoutRequest = array(
            "checkout" => array(
                "line_items"               => $lineItems,
                "shipping_address"         => $shippingAddress,
                "email"                    => !empty($receivedData['checkout']['email']) ? $receivedData['checkout']['email'] : '',
                "external_checkout_id"     => !empty($receivedData['checkout']['external_checkout_id'])
                    ? $receivedData['checkout']['external_checkout_id'] : '',
                "external_checkout_source" => !empty($receivedData['checkout']['external_checkout_source'])
                    ? $receivedData['checkout']['external_checkout_source'] : '',
                "discount_code"            => !empty($receivedData['checkout']['discount_code'])
                    ? $receivedData['checkout']['discount_code'] : '',
            )
        );

        foreach ($checkoutRequest['checkout'] as $key => $value) {
            if (empty($value)) {
                unset($checkoutRequest['checkout'][$key]);
            }
        }

        Log::channel('checkoutLog')->info("Checkout request: ".json_encode($checkoutRequest));

        $checkoutReturn = $this->modelObj->createCheckout(json_encode($checkoutRequest));

        Log::channel('checkoutLog')->info("Checkout response: ".json_encode($checkoutReturn));

        if (isset($checkoutReturn->error) || isset($checkoutReturn->errors) ||
            empty($checkoutReturn)) {

            $status = "failed";
            $errorMessage  = "Unknown error";
            if (isset($checkoutReturn->error)) {

                $errorMessage = json_encode($checkoutReturn->error);

            } else if (isset($checkoutReturn->errors)) {

                $errorMessage = json_encode($checkoutReturn->errors);
            } else if (empty($checkoutReturn)) {

                $errorMessage = "The response from the recharge API is empty";
            }

            $status = "Failed";
            $error = "Bad response";
            $message = "Failed to create the Checkout";

            Log::channel('checkoutLog')->info("Failed to create the Checkout, Error:".$errorMessage);
            return $this->returnData($status, $error, $message, $request->input('callback'));
        }

        $url = env('EQUELLE_CHECKOUT_URL', true) . $checkoutReturn->checkout->token .
            '?myshopify_domain=' . env('SHOPIFY_DOMAIN', true);

        echo $request->input('callback') . '({"status":"success","url": "' . $url . '",
            "errorMessage":""})';
        die();
    }

    /**
     * Response function
     */
    public function returnData($status, $error, $message, $jsonCallback = null) {

        $returnData = array(
            'status' => $status,
            'error' => $error,
            'message' => $message,
        );
        if (!empty($jsonCallback)) {

            echo $jsonCallback . '(' . json_encode($returnData) . ')';
            die();
        } else {

            return response()->json($returnData);
            die();
        }
    }

    /**
     * Function to get all shopify products
    */
    public function getShopifyProductDetails() {

        $shopifyProducts  = [];
        $pageNo = 1;
        $limit  = 200;
        do {
            try {
                if (!empty($products['link'])) {

                    $products = $this->productObj->getShopifyProducts($limit,$products['link']);

                 } else {

                    $products = $this->productObj->getShopifyProducts($limit);
                }
            }
            catch (Exception $e) {

                Log::channel('checkoutLog')->info("Error form Shopify when fetching the product details");
                // echo PHP_EOL."Error form shopify when fetching the
                //             product details and error is -  ".$e->getMessage();
                die;
            }

            $productsToSkip = array('4390536118407', '4494142210186');
            
            if (!empty($products) && isset($products)) {
                
                foreach ($products['body'] as $product) {

                    if (in_array($product['id'], $productsToSkip)) {
                        continue;
                    }

                    if (!empty($product['variants'])) {

                        $variantDetails = [];
                        foreach ($product['variants'] as $variant) {

                            if (!empty($variant['id'])) {
                                $variantDetails = [
                                    'variant_id' => $variant['id'],
                                    'product_id' => $variant['product_id'],
                                    'variant_title' => strip_tags($variant['title']),
                                    'product_title' => strip_tags($product['title']),
                                    'sku' => $variant['sku'],
                                    'price' => $variant['price']
                                ];
                                $shopifyProducts[$variant['id']] = $variantDetails;
                            }
                        }
                    }
                }
                $productCount = count($products['body']);
            }

            $pageNo++;

            if (empty($products['link'])) {
                break;
            }
        } while($productCount == $limit);

        return $shopifyProducts;
    }
}
