<?php
namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");

use App\Models\RechargeApi;
use Illuminate\Http\Request;
use Config;
use Log;

class CouponValidityController extends Controller 
{
    private $rechargeObj;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $api;

    public function __construct() {      
        
        $this->rechargeObj = new RechargeApi();

        date_default_timezone_set('US/Eastern');
    }
   
    /**
     * Checking discount code validity
     */
    public function checkCodeValidity(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {

            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('couponCodeValidityCheckingLog')->info("Endpoint parameters are missing");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }

        $receivedData = json_decode($requestData);

        if (!isset($receivedData->discount_code) || empty($receivedData->discount_code)) {

            $message = "Missing required parameter discount_code";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('couponCodeValidityCheckingLog')->info("Missing required parameter discount_code");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }

        $discountCode = trim($receivedData->discount_code);

        $discountDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);
       
        if (!isset($discountDetails) || !isset($discountDetails->discounts) || isset($discountDetails->error)
         || isset($discountDetails->errors) || empty($discountDetails->discounts) || empty($discountDetails->discounts[0])) {

            $status = "Failed";
            $error  = "Unknown error";
            if (isset($discountDetails->error)) {
                $error = json_encode($discountDetails->error);
            } else if (isset($discountDetails->errors)) {
                $error = json_encode($discountDetails->errors);
            } else {
                $error = "Failed to get the discount code details";
            }
            $message = "Failed to get the discount details from recharge";

            Log::channel('couponCodeValidityCheckingLog')->info(
                "Failed to get the discount details from recharge: ".
                " | discountCode -> ".$discountCode.
                " | Error -> ".$error
            );

            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
        
        $endDate = !empty($discountDetails->discounts[0]->ends_at) ? trim($discountDetails->discounts[0]->ends_at) : null;
        if ($endDate == null) {
            $status = "Valid";
            $message = "The coupon is valid.";
            $error  = "";

            Log::channel('couponCodeValidityCheckingLog')->info(
                "The coupon is valid. ".
                " | discountCode -> ".$discountCode.
                " | Error -> ".$error
            );

        } else {
            $currentDate = date('Y-m-d h:i:s');

            if (strtotime($currentDate) > strtotime($endDate)) {
                $status = "Invalid";
                $message = "The coupon is expired.";
                $error  = "";

                Log::channel('couponCodeValidityCheckingLog')->info(
                    "The coupon is expired. ".
                    " | discountCode -> ".$discountCode.
                    " | Error -> ".$error
                );

            } else {
                $status = "Valid";
                $message = "The coupon is valid.";
                $error  = "";

                Log::channel('couponCodeValidityCheckingLog')->info(
                    "The coupon is valid. ".
                    " | discountCode -> ".$discountCode.
                    " | Error -> ".$error
                );
            }
        }

        $response = array(
            "status" => $status,
            "message" => $message,
            "error" => $error
        );
        echo $request->input('callback') . '(' . json_encode($response) . ')';
        die();
    }

}
