<?php
namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");

use Illuminate\Http\Request;
use Config;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;
use Log;


class CustomerController extends Controller {

    private $helper;
    private $shopify_api_version;
    private $api;

    public function __construct() {      
        
        $options = new Options();
        $options->setVersion(env('SHOPIFY_API_VERSION', true));

        // Create the client and session
        $this->api = new BasicShopifyAPI($options);
        $shDomain = env('SHOPIFY_DOMAIN', true);
        $accessToken = env('SHOPIFY_API_PASSWORD', true);
        $this->api->setSession(new Session($shDomain, $accessToken));
    }


    /**
     * Get customer standing status in Shopify
     */
    public function getCustomerStanding(Request $data) {
        
        $receivedData = !empty($data['data']) ? $data['data'] : "";
        if (empty($receivedData)) {

            $message = "Endpoint parameters are missing";
            $returnData = [
                'message' => $message
            ];
            Log::channel('customerControllerLog')->info("Endpoint parameters are missing");
            $this->returnResponse(600, "error", $returnData, $data);
        }
        
        Log::channel('customerControllerLog')->info("Request: ".$receivedData);
        $receivedData = json_decode($receivedData);

        $requestEmail = !empty($receivedData->email) ? trim($receivedData->email) : "";
        $currentOrderId = !empty($receivedData->orderId) ? trim($receivedData->orderId) : "";
        $customerAvailable = false;
        
        if (!isset($requestEmail) || empty($requestEmail)) {
            
            $returnData = [
                'message' => 'Parameter missing - email'
            ];
            Log::channel('customerControllerLog')->info("Parameter missing - email");
            $this->returnResponse(600, "error", $returnData, $data);
        }

         if (!isset($currentOrderId) || empty($currentOrderId)) {
            
            $returnData = [
                'message' => 'Parameter missing - orderId'
            ];
            Log::channel('customerControllerLog')->info("Parameter missing - orderId");
            $this->returnResponse(600, "error", $returnData,$data );
        }
        
        $gQuery = '{
                      customers(first:250, query:"'.$requestEmail.'") {
                        edges {
                          node {
                            email
                          }
                        }
                      }
                   }';
        
        $customerResult = $this->api->graph($gQuery);

        if (!isset($customerResult['body']['data']['customers']['edges'][0]) || empty($customerResult['body']['data']['customers']['edges'][0])) {
            $returnData = [
                'message' => 'No such customer found on Shopify',
                "customerStanding" => null 
            ];
            Log::channel('customerControllerLog')->info("No such customer found on Shopify");
            $this->returnResponse(200, "success", $returnData, $data);

        } else {

            $customers = $customerResult['body']['data']['customers']['edges'];   
                        
            foreach ($customers as $customer) {
                if (trim(strtolower($customer['node']['email'])) == strtolower($requestEmail)) {
                    $customerAvailable = true;
                }
            }
            if (!$customerAvailable) {
                $returnData = [
                    'message' => 'No such customer found on Shopify',
                    "customerStanding" => null 
                ];
                Log::channel('customerControllerLog')->info("No such customer found on Shopify");
                $this->returnResponse(200, "success", $returnData,$data );
            }
        }
        
        $gOQuery = '{
                      orders(first:250, query:"email:'.$requestEmail.' AND status:any") {
                        edges {
                          node {
                            id
                            email
                          }
                        }
                      }
                   }';
        
        
        $customerOrderResult = $this->api->graph($gOQuery);
        
        if($customerAvailable && (!isset($customerOrderResult['body']['data']['orders']['edges'][0]) || empty($customerOrderResult['body']['data']['orders']['edges'][0]))) {
            
            $returnData = [
                "customerStanding" => 'New'                           
            ];
            Log::channel('customerControllerLog')->info("Found customer in Shopify, customerStanding: New");
            $this->returnResponse(200, "success", $returnData,$data );;
        }
        else {
            $orders = $customerOrderResult['body']['data']['orders']['edges'];  
            $orderDetails = [];
            foreach ($orders as $order) {
                if ($order['node']['email'] == $requestEmail) {
                    $orderId = explode('/',$order['node']['id']);
                    $orderId = end($orderId);
                    $orderDetails[] = trim($orderId);
                }
            }
            

            if (count($orderDetails) > 1) {
                $returnData = [
                    "customerStanding" => 'Returning'                           
                ];
                Log::channel('customerControllerLog')->info("Found customer in Shopify, customerStanding: Returning");
                $this->returnResponse(200, "success", $returnData, $data);
            }    
            
            if (count($orderDetails) == 1 && in_array($currentOrderId, $orderDetails)) {
                $returnData = [
                    "customerStanding" => 'New'                           
                ];
                Log::channel('customerControllerLog')->info("Found customer in Shopify, customerStanding: New");
                $this->returnResponse(200, "success", $returnData, $data);
            } else {
                $returnData = [
                    "customerStanding" => 'New'                           
                ];
                Log::channel('customerControllerLog')->info("Found customer in Shopify, customerStanding: New");
                $this->returnResponse(200, "success", $returnData, $data);
            }
        }
    }


    public function returnResponse($status,$message, $body, $request) {
        $response = array(
            "status"  => $status,
            "message" => $message,
            "body" => $body
        );
        echo $request->input('callback') . '(' . json_encode($response) . ')';
        die();
    }

}
