<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Jobs\IterableSubscriptionCreateEvent;
use App\Jobs\IterableSubscriptionCancelEvent;
use App\Jobs\IterableSubscriptionChargeUpcomingEvent;
use App\Jobs\IterableOnetimeCreateEvent;
use Log;

class IterableWebhookController extends Controller
{
    public $rechargeWebhookSecret;
    private $logDirectory;
    private $logFiles;
    private $logPath;

    public function __construct() {

        $this->rechargeWebhookSecret = env('RECHARGE_API_CLIENT_SECRET', true);
    }

    /**
     * Subscription created event creation
     */
    public function subscriptionCreatedEventWebhook() {

        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verifyRechargeWebhook($requestData, $received_digest);

        if ($verified) {
            Log::channel('iterableWebhookLog')->info("subscriptionCreatedEventWebhook: Request => ".$requestData);
            $receivedData = json_decode($requestData);
            if (empty($receivedData->subscription->id)) {
                Log::channel('iterableWebhookLog')->info("subscriptionCreatedEventWebhook: Error in the webhook response
                (Created subscription) - Missing required parameters in the webhook response or the response is not correct");
            } else {
                $response = dispatch(new IterableSubscriptionCreateEvent($receivedData));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {
            Log::channel('iterableWebhookLog')->info("subscriptionCreatedEventWebhook: Recharge subscription create Webhook 
            validation failed");
        }
    }

    /**
     * Subscription cancelled event creation
     */
    public function subscriptionCanceledEventWebhook() {

        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verifyRechargeWebhook($requestData, $received_digest);

        if ($verified) {
            Log::channel('iterableWebhookLog')->info("subscriptionCanceledEventWebhook: Request => ".$requestData);
            $receivedData = json_decode($requestData);
            if (empty($receivedData->subscription->id)) {
                Log::channel('iterableWebhookLog')->info("subscriptionCanceledEventWebhook: Error in the webhook response
                (Cancel subscription) - Missing required parameters in the webhook response or the response is not correct");
            } else {
                $response = dispatch(
                    new IterableSubscriptionCancelEvent($receivedData->subscription->customer_id)
                );
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {
            Log::channel('iterableWebhookLog')->info("subscriptionCanceledEventWebhook: Recharge subscription cancelled Webhook 
            validation failed");
        }
    }

    /**
     * Upcoming charge event creation
     */
    public function upcomingChargesEventWebhook() {

        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verifyRechargeWebhook($requestData, $received_digest);

        if ($verified) {
            Log::channel('iterableWebhookLog')->info("upcomingChargesEventWebhook: Request => ".$requestData);
            $receivedData = json_decode($requestData, true);
            if (empty($receivedData['charge']['id'])) {
                Log::channel('iterableWebhookLog')->info("upcomingChargesEventWebhook: Error in the webhook response
                (upcoming charge) - Missing required parameters in the webhook response or the response is not correct");
                
            } else {
                $response = dispatch(new IterableSubscriptionChargeUpcomingEvent($receivedData));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {
           Log::channel('iterableWebhookLog')->info("upcomingChargesEventWebhook: Recharge subscription cancelled Webhook 
           validation failed");
        }
    }
    
    /**
     * Order create event creation
     */
    public function orderCreateEventWebhook() {

        $received_digest = $_SERVER['HTTP_X_RECHARGE_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verifyRechargeWebhook($requestData, $received_digest);
        if ($verified) {
            Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: Request => ".$requestData);
            $receivedData = json_decode($requestData);
            if(empty($receivedData->order->id)) {
                Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: Error in the webhook response(Created OTP) - 
                Missing required parameters in the webhook response or the response is not correct");
            } else {
                if($receivedData->order->tags){
                    $tags = explode(',', $receivedData->order->tags);
                    if(!empty($tags) && in_array('OTP', $tags)){
                        Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: OTP order received => ".$receivedData->order->id);
                    	$queueResponse = dispatch(new IterableOnetimeCreateEvent($receivedData));
                		ignore_user_abort(true);
                        header("Content-Length: 0");
                        header("Connection: Close");
                        flush();
                    } else {
                        Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: Subscription order received => ".$receivedData->order->id);
                    }
                }
            }
        } else {
            Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: Recharge order creation Webhook validation failed");
        }
    }

    public function verifyRechargeWebhook($data, $received_digest) {

        $calculated_digest = hash('sha256', $this->rechargeWebhookSecret.$data);
        if (hash_equals($received_digest, $calculated_digest)) {
            return true;
        } else {
            return false;
        }
    }
}
