<?php

namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");
use App\Models\RechargeApi;
use App\Models\IterableApi;
use Illuminate\Http\Request;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;
use Log;


class SubscriptionController extends Controller
{
    private $rechargeObj;
    private $iterableObj;
    private $logDirectory;
    private $logFiles;
    private $logPath;
    private $api;

    public function __construct() {

        $this->rechargeObj   = new RechargeApi();
        $this->iterableObj   = new IterableApi();

        // Create options for the API
        $options = new Options();
        $options->setVersion(env('SHOPIFY_API_VERSION', true));

        // Create the client and session
        $this->api = new BasicShopifyAPI($options);
        $this->api->setSession(new Session(env('SHOPIFY_DOMAIN', true), env('SHOPIFY_API_PASSWORD', true)));
    }

    /**
     * Reactivate cancelled recharge subscription with coupon
    */
    public function reactivateSubscriptionWithCoupon(Request $request) {

        $requestData = !empty($request->input('data')) ? trim($request->input('data')) : "";
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: Endpoint parameters are missing");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: Request => ".$requestData);
        $receivedData = json_decode($requestData);
        $subscriptionId = !empty($receivedData->subscriptionId) ? trim($receivedData->subscriptionId) : "";
        $customerHash = !empty($receivedData->customerHash) ? trim($receivedData->customerHash) : "";

        if (empty($subscriptionId)) {
            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter subscription_id";
            $status = "Failed";
            $response = array(
                "status" => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: Missing required parameter subscription_id");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
        if (empty($customerHash)) {
            $message = "Failed to reactivate the subscription";
            $error = "Missing required parameter recharge customer hash";
            $status = "Failed";
            $response = array(
                "status" => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: Missing required parameter recharge customer hash");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }

        $subscriptionDetails = $this->rechargeObj->getSubscription($subscriptionId);

        if (!isset($subscriptionDetails) || isset($subscriptionDetails->error)
         || isset($subscriptionDetails->errors) || empty($subscriptionDetails->subscription->customer_id)) {

            $status = "Failed";
            $error  = "Unknown error";
            if (isset($subscriptionDetails->error)) {
                $error = json_encode($subscriptionDetails->error);
            } else if (isset($subscriptionDetails->errors)) {
                $error = json_encode($subscriptionDetails->errors);
            } else {
                $error = "Failed to get the subscription details by Id";
            }
            $message = "Failed to get the subscription details";

            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: ".
                "Failed to get the subscription details: ".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId.
                " | Error -> ".$error
            );

            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($subscriptionDetails->subscription->customer_id);

        if (!isset($customerDetails) || isset($customerDetails->error)
         || isset($customerDetails->errors) || empty($customerDetails->customer->hash)) {

            $status = "Failed";
            $error  = "Unknown error";
            if (isset($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } else if (isset($customerDetails->errors)) {
                $error = json_encode($customerDetails->errors);
            } else {
                $error = "Failed to get the customer details";
            }
            $message = "Failed to get the customer details from recharge";

            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: ".
                "Failed to get the customer details from recharge: ".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId.
                " | Error -> ".$error
            );

            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }

        $rechargeCustomerHash = trim($customerDetails->customer->hash);

        if ($customerHash == $rechargeCustomerHash) {

            $reactivateResponse = $this->rechargeObj->activateSubscription($subscriptionId);

            if (isset($reactivateResponse->error) || isset($reactivateResponse->errors) || empty($reactivateResponse)) {
                $status = "Failed";
                $error  = "Unknown error";
                if (isset($reactivateResponse->error)) {
                    $error = json_encode($reactivateResponse->error);
                } else if (isset($reactivateResponse->errors)) {
                    $error = json_encode($reactivateResponse->errors);
                } else {
                    $error = "Failed to reactivate the subscription";
                }

                $message = "Failed to reactivate the subscription";
                $status = "Failed";

                Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: ".
                    "Failed to reactivate the subscription: ".
                    " | customerHash -> ".$customerHash.
                    " | Subscription -> ".$subscriptionId.
                    " | Error -> ".$error
                );

                $response = array(
                    "status" => $status,
                    "message" => $message,
                    "error" => $error
                );
                echo $request->input('callback') . '(' . json_encode($response) . ')';
                die();
            } else {


                $subscriptionChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($subscriptionId);

                    if (!empty($subscriptionChargeDetails->error) || !empty($subscriptionChargeDetails->errors) ||
                        empty($subscriptionChargeDetails)) {

                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($subscriptionChargeDetails->error)) {
                            $error = json_encode($subscriptionChargeDetails->error);
                        } else if (!empty($subscriptionChargeDetails->errors)) {
                            $error = json_encode($subscriptionChargeDetails->errors);
                        } else {
                            $error = "Failed to get the charge details by using subscriptionId: ".$subscriptionId;
                        }

                        $message = "Failed to get the subscription charge details from recharge";

                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Failed to get the charge details by using subscriptionId:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId.
                            " | Error -> ".$error
                        );
                        
                        $response = array(
                            "status"       => $status,
                            "message" => $message,
                            "error" => $error
                        );
                        echo $request->input('callback') . '(' . json_encode($response) . ')';
                        die();
                    }

                $subscriptionChargeStatus = null;
                $subscriptionChargeID = null;
                $queuedCharge = [];
                foreach ($subscriptionChargeDetails->charges as $chargeValue) {

                    $subscriptionChargeTags = strtolower(trim($chargeValue->tags));
                    if (strpos($subscriptionChargeTags, 'subscription' ) !== false &&
                    strtolower(trim($chargeValue->status)) == 'queued' ) {

                        $subscriptionChargeID = $chargeValue->id;
                        $subscriptionChargeStatus = $chargeValue->status;
                        $queuedCharge = $chargeValue;
                    }
                }

                /* Iterable event for reactivating the subscription*/
                $subscribedItems = '';

                if(!empty($queuedCharge)){
                    $subscribedItems = $queuedCharge->line_items;
                }

                if (!empty($customerDetails->customer->email)) {

                    $lineItemDetails = [];
                    if(!empty($subscribedItems)){
                        foreach($subscribedItems as $item) {

                            $lineItem = new \stdClass();

                            $lineItem->variant_id = $item->shopify_variant_id;
                            $lineItem->quantity = $item->quantity;
                            $lineItem->sku = $item->sku;
                            $lineItem->price = $item->price;
                            $lineItem->title = $item->title;
                            $lineItemDetails[] = $lineItem;

                        }
                    }

                    $customerEmail = $customerDetails->customer->email;

                    $eventRequest = array(
                        "email" => strtolower($customerEmail),
                        "eventName" => "subscriptionReactivated",
                        "dataFields" => array(
                            "lineItems" => $lineItemDetails
                        )
                    );

                    $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

                    if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Subscription reactivation event - Successfully created for Email (".
                            strtolower($customerEmail).")".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId
                        );
                    } else {
                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Subscription reactivation event - Failed to create event for Email (".
                            strtolower($customerEmail).")".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId
                        );
                    }
                }
                /* Iterable event for reactivating the subscription*/

                /* apply coupon for next charge */
                if (isset($receivedData->couponCode) && !empty($receivedData->couponCode)) {

                    if (empty($subscriptionChargeStatus)) {

                        $message = "The subscription queued charge was not found to apply the discount, subscriptionID: "
                        .$subscriptionId;
                        $error = "Queued charge not found";
                        $status = "Failed";

                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Queued charge not found:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId
                        );
                        
                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $request->input('callback') . '(' . json_encode($response) . ')';
                        die();
                    }

                    $discountCode = trim($receivedData->couponCode);
                    $discountCodeDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);
                    if (!empty($discountCodeDetails->error) || !empty($discountCodeDetails->errors) ||
                    empty($discountCodeDetails)) {

                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountCodeDetails->error)) {
                            $error = json_encode($discountCodeDetails->error);
                        } else if (!empty($discountCodeDetails->errors)) {
                            $error = json_encode($discountCodeDetails->errors);
                        } else {
                            $error = "Failed to get the discount code details from recharge";
                        }

                        $message = "Failed to get the discount details from recharge by using the provided discountCode: "
                        .$discountCode;

                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Failed to get the discount details from recharge by using the provided discountCode:".
                            " | customerHash -> ".$customerHash.
                            " | Subscription -> ".$subscriptionId.
                            " | DiscountCode -> ".$discountCode.
                            " Error -> ".$error
                        );
                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $request->input('callback') . '(' . json_encode($response) . ')';
                        die();
                    }
                    $discountRequest = array(
                        "discount_code" => $discountCode
                    );

                    $discountApplyResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($subscriptionChargeID,
                    json_encode($discountRequest));

                    if (!empty($discountApplyResponse->error) || !empty($discountApplyResponse->errors) ||
                    empty($discountApplyResponse)) {

                        $status = "Failed";
                        $error  = "Unknown error";
                        if (!empty($discountApplyResponse->error)) {
                            $error = json_encode($discountApplyResponse->error);
                        } else if (!empty($discountApplyResponse->errors)) {
                            $error = json_encode($discountApplyResponse->errors);
                        } else {
                            $error = "Failed to apply the discount to the charge";
                        }

                        $message = "Failed to apply the discount to the charge chargeID: ".$subscriptionChargeID;

                        Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                        Failed to apply the discount to the charge: ".
                            " | Subscription -> ".$subscriptionId.
                            " | customerHash -> ".$customerHash.
                            " | chargeID: ".$subscriptionChargeID.
                            " | DiscountCode: ".$discountCode.
                            " | Error -> ".$error
                        );
                        $response = array(
                            "status"  => $status,
                            "message" => $message,
                            "error"   => $error
                        );
                        echo $request->input('callback') . '(' . json_encode($response) . ')';
                        die();
                    }


                    $message = "Successfully activated the subscription and applied the discount";
                    $error = "";
                    $status = "Success";

                    Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                    Successfully activated the subscription and applied the discount.".
                        " | Subscription -> ".$subscriptionId.
                        " | customerHash -> ".$customerHash.
                        " | chargeID: ".$subscriptionChargeID.
                        " | DiscountCode: ".$discountCode
                    );

                    $response = array(
                        "status"  => $status,
                        "message" => $message,
                        "error"   => $error
                    );
                    echo $request->input('callback') . '(' . json_encode($response) . ')';
                    die();
                }

                /* apply coupon for next charge */
                Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
                Subscription reactivated successfully.".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId
                );
                
                $message = "Subscription reactivated successfully";
                $error = "";
                $status = "Success";
                $response = array(
                        "status"  => $status,
                        "message" => $message,
                        "error"   => $error
                );
                echo $request->input('callback') . '(' . json_encode($response) . ')';
                die();
            }

        } else {
            $message = "Failed to reactivate the subscription";
            $error = "Customer hash tags are not matched";
            $status = "Failed";

            Log::channel('subscriptionControllerLog')->info("reactivateSubscriptionWithCoupon: 
            Failed to reactivate the subscription. Customer hashtags are not matched:".
                " | customerHash -> ".$customerHash.
                " | Subscription -> ".$subscriptionId
            );
            $response = array(
                "status"  => $status,
                "message" => $message,
                "error"   => $error
            );
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
    }
    
    
    /**
     * Convert OTP to subscription
     */
    public function otpToSubscription(Request $data) {

        $receivedData = !empty($data->input('data')) ? trim($data->input('data')) : "";
        if (empty($data['data'])) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Endpoint parameters are missing");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        Log::channel('subscriptionControllerLog')->info("otpToSubscription: Request => ".$receivedData);
        $receivedData = json_decode($receivedData);
        if (empty($receivedData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Endpoint parameters are missing");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $emptyKey = null;
        $requiredRequestKeys = ["address_id", "hash_value"];
        $requestKeys = [];
        $valueStatus = 0;
        foreach ($receivedData as $key => $receiveValue) {

            $requestKeys[] = $key;
            if (empty($receiveValue) && $valueStatus == 0) {
                $emptyKey = $key;
                $valueStatus = 1;
            }
        }
        $keyCompareStatus = array_diff($requiredRequestKeys, $requestKeys);
        if(!empty($keyCompareStatus)) {
            $missedParameters = implode(",",$keyCompareStatus);
            $message = $missedParameters." endpoint parameters are missing";
            $error = "Some parameters are missing in the endpoint request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Some parameters are missing in the 
            endpoint request");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        if (!empty($emptyKey)) {
            $message = $emptyKey." is missing";
            $error = "Bad request";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Parameters are missing in the request");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        
        $discountCode = null;
        $addressID = trim($receivedData->address_id);
        $otpChargeDetails = $this->rechargeObj->getOtpChargeDetails($addressID);
        if (!empty($otpChargeDetails->error) || !empty($otpChargeDetails->errors) ||
            empty($otpChargeDetails)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($otpChargeDetails->error)) {
                $error = json_encode($otpChargeDetails->error);
            } else if (!empty($otpChargeDetails->errors)) {
                $error = json_encode($otpChargeDetails->error);
            } else {
                $error = "Failed to get the charge details by using addressID:".$addressID;
            }

            $message = "Failed to get the charge details from recharge";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Error => ".$error);
            $this->requestResponse("Failed", $message, $error, $data->input('callback'));
        }
        $customerHash = strtolower(trim($receivedData->hash_value));
        $originalCustomerHash = null;
        $customerEmail = null;
        $username = null;
        $rechargeChargeDetails = null;
        $subscriptionExistingStatus = null;
        foreach ($otpChargeDetails->charges as $chargeValue) {

            if ($addressID == trim($chargeValue->address_id) &&
            $customerHash == strtolower(trim($chargeValue->customer_hash))) {

                $originalCustomerHash = $chargeValue->customer_hash;
                $customerEmail = $chargeValue->email;
                $username  = !empty($chargeValue->first_name) ? $chargeValue->first_name.' ' : '';
                $username .= !empty($chargeValue->last_name) ? $chargeValue->last_name : '';
                $chargeTags = strtolower(trim($chargeValue->tags));
                if (strpos($chargeTags, 'otp' ) !== false) {
                    $rechargeChargeDetails = $chargeValue;
                }
                if (strpos($chargeTags, 'subscription' ) !== false &&
                strtolower(trim($chargeValue->status)) == 'queued' ) {
                    $subscriptionExistingStatus = $chargeValue->status;
                }
            }
        }
        if (empty($originalCustomerHash)) {

            $message = "Customer hash value is not matching";
            $error = "Security breach";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Customer hash value is not matching");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        if (!empty($subscriptionExistingStatus)) {

            $message = "There is already a queued charge in the given addressID:".$addressID;
            $error = "Existing an active queued charge";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Already existing an active queued charge");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $chargeTags = strtolower(trim($otpChargeDetails->charges[0]->tags));
        if (empty($rechargeChargeDetails)) {
            $message = "The charge in the given addressID is not an OTP charge";
            $error = "OTP charge not found";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: OTP charge not found");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $otpLineItemQuantity = [];
        $lineItemProperties = [];
        $chargeVariantDetails = [];

        // $i = 0;
        $shopifyVariantId = '';
        $shopifyVariantQuantity = '';
        $shopifyVariantPrice = '';
        $shopifyVariantSku = '';
        $shopifyProductId = '';
        foreach ($rechargeChargeDetails->line_items as $item) {
            $shopifyVariantId = $item->shopify_variant_id;
            $shopifyVariantQuantity = $item->quantity;
            $shopifyVariantPrice = $item->price;
            $shopifyVariantSku      = $item->sku;
            $shopifyProductId = $item->shopify_product_id;
        }
        if (empty($shopifyVariantId)) {

            $message = "The line items in the OTP charge is empty addressID:".$addressID;
            $error = "Line items not found in the OTP charge";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Line items not found in the OTP charge");
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        
        $lineItemProperties[] = (object)array(
            "name" => "SKU",
            "value" => $shopifyVariantSku
        );

        $lineItemProperties[] = (object)array(
            "name" => "charge_interval_frequency",
            "value" => 27
        );
        $lineItemProperties[] = (object)array(
            "name" => "shipping_interval_frequency",
            "value" => 27
        );
        $lineItemProperties[] = (object)array(
            "name" => "shipping_interval_unit_type",
            "value" => "day"
        );
        

        $variantErrorStatus = 0;
        $subscriptionPrice = null;
        
        $variantAPiResponse = $this->api->rest('GET', '/admin/variants/'.$shopifyVariantId.'.json');
        
        if ( !empty($variantAPiResponse['errors']) || !empty($variantAPiResponse['error'])) {
            
            $variantError = !empty($variantAPiResponse['body']) ? json_encode($variantAPiResponse['body']) :
            json_encode($variantAPiResponse['status']);
            
            $message = "The variantID in the endpoint request is not valid variantID: ".$shopifyVariantId.", Error: ".$variantError;
            $error = "Invalid variantID found";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $variantDetails = $variantAPiResponse['body'];
        
        $subscriptionPrice = floatval($subscriptionPrice) +
                    (floatval($variantDetails['variant']['price']) * intval($shopifyVariantQuantity));
        
        
        // $subscriptionPrice = floatval($subscriptionPrice) + (floatval($shopifyVariantPrice) * intval($shopifyVariantQuantity));

        if ($variantErrorStatus == 1) {

            $message = "The variantID in the endpoint request is not valid variantID: ".$variantMismatchID.", Error: ".$variantError;
            $error = "Invalid variantID found";
            $status = "Failed";
            $this->requestResponse($status, $message, $error, $data->callback);
        }
        $subscriptionSKU = "vitamin-pack";
        $today = time();
        $todayDate = date('Y-m-d');
        $defaultSubscriptionPeriod = 27;
        $otpChargeProcessedDate = date('Y-m-d', strtotime($rechargeChargeDetails->processed_at));
        $otpChargeProcessedDate = strtotime($otpChargeProcessedDate);
        $dateDiff = $today - $otpChargeProcessedDate;
        $dateDiffDays = round($dateDiff / (60 * 60 * 24));
        $nextChargePeriod = $defaultSubscriptionPeriod - $dateDiffDays;
        if ($nextChargePeriod <= 0) {
            $nextChargeDate = $todayDate;
        } else {
            $nextChargeDate = date('Y-m-d',strtotime('+'.$nextChargePeriod.' days',strtotime($todayDate)));
        }
        $intervalUnit = "day";

        $subscriptionRequest = array(
            "address_id" => $rechargeChargeDetails->address_id,
            "next_charge_scheduled_at" => $nextChargeDate,
            //"next_charge_scheduled_at" => $todayDate,
            "quantity" => !empty($rechargeChargeDetails->line_items[0]->quantity) ?
            $rechargeChargeDetails->line_items[0]->quantity : 1,
            "price" => floatval($subscriptionPrice),
            //"sku" => $subscriptionSKU,
            "shopify_variant_id" => $shopifyVariantId,
            "order_interval_unit" => $intervalUnit,
            "order_interval_frequency" => $defaultSubscriptionPeriod,
            "charge_interval_frequency" => $defaultSubscriptionPeriod,
            "properties" => $lineItemProperties
        );
        $subscriptionCreateResponse = $this->rechargeObj->createSubscription(json_encode($subscriptionRequest));
        if (!empty($subscriptionCreateResponse->error) || !empty($subscriptionCreateResponse->errors) ||
            empty($subscriptionCreateResponse)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionCreateResponse->error)) {
                $error = json_encode($subscriptionCreateResponse->error);
            } else if (!empty($subscriptionCreateResponse->errors)) {
                $error = json_encode($subscriptionCreateResponse->errors);
            } else {
                $error = "Failed to create the subscription under the addressID:".$addressID;
            }
            $message = "Failed to create the subscription in recharge";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Error => ".$error);
            $this->requestResponse("Failed", $message, $error, $data->input('callback'));
        }
        // Create iterable event convertToSubscription
        $eventRequest = array(
            "email" => strtolower($customerEmail),
            "eventName" => "convertToSubscription",
            "dataFields" => array(
                "userName" => $username
            )
        );

        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: convertToSubscription iterable event - 
            Successfully created event for Email (".strtolower($customerEmail).")");
        } else {
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: convertToSubscription iterable event - 
            Failed to create event for Email (".strtolower($customerEmail).")");
        }
        // Applying the discount to the created subscription charge
        $subscriptionID = $subscriptionCreateResponse->subscription->id;
        $subscriptionChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($subscriptionID);
        if (!empty($subscriptionChargeDetails->error) || !empty($subscriptionChargeDetails->errors) ||
            empty($subscriptionChargeDetails)) {

            $status = "failed";
            $error  = "Unknown error";
            if (!empty($subscriptionChargeDetails->error)) {
                $error = json_encode($subscriptionChargeDetails->error);
            } else if (!empty($subscriptionChargeDetails->errors)) {
                $error = json_encode($subscriptionChargeDetails->errors);
            } else {
                $error = "Failed to get the charge details by using subscriptionID: ".$subscriptionID;
            }

            $message = "Failed to get the subscription charge details from recharge";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Error => ".$error);
            $this->requestResponse("Failed", $message, $error, $data->input('callback'));
        }
        $subscriptionChargeStatus = null;
        $subscriptionchargeID = null;
        foreach ($subscriptionChargeDetails->charges as $chargeValue) {

            $subscriptionChargeTags = strtolower(trim($chargeValue->tags));
            if (strpos($subscriptionChargeTags, 'subscription' ) !== false &&
            strtolower(trim($chargeValue->status)) == 'queued' ) {

                $subscriptionchargeID = $chargeValue->id;
                $subscriptionChargeStatus = $chargeValue->status;
            } 
        }
        if (empty($subscriptionChargeStatus)) {

            $message = "The subscription queued charge not found for apply the discount subscriptionID: "
            .$subscriptionID;
            $error = "Queued charge not found";
            $status = "Failed";
            Log::channel('subscriptionControllerLog')->info("otpToSubscription: Queued charge not found, Error => ".$message);
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        if (!empty($receivedData->discountCode)) {
            $discountCode = trim($receivedData->discountCode);
            $discountCodeDetails = $this->rechargeObj->getDiscountDetailsByDiscountCode($discountCode);
            if (!empty($discountCodeDetails->error) || !empty($discountCodeDetails->errors) ||
            empty($discountCodeDetails)) {

                $status = "failed";
                $error  = "Unknown error";
                if (!empty($discountCodeDetails->error)) {
                    $error = json_encode($discountCodeDetails->error);
                } else if (!empty($discountCodeDetails->errors)) {
                    $error = json_encode($discountCodeDetails->errors);
                } else {
                    $error = "Failed to get the discount code details from recharge";
                }

                $message = "Failed to get the discount details from recharge by using the provided discountCode: "
                .$discountCode;
                Log::channel('subscriptionControllerLog')->info("otpToSubscription: Error => ".$error);
                $this->requestResponse("Failed", $message, $error, $data->input('callback'));
            }
            $discountRequest = array(
                "discount_code" => $discountCode
            );
            $discountApplyResponse = $this->rechargeObj->applyDiscountToChargeByDiscountCode($subscriptionchargeID, 
            json_encode($discountRequest));
            if (!empty($discountApplyResponse->error) || !empty($discountApplyResponse->errors) ||
            empty($discountApplyResponse)) {

                $status = "failed";
                $error  = "Unknown error";
                if (!empty($discountApplyResponse->error)) {
                    $error = json_encode($discountApplyResponse->error);
                } else if (!empty($discountApplyResponse->errors)) {
                    $error = json_encode($discountApplyResponse->errors);
                } else {
                    $error = "Failed to apply the discount to the charge";
                }

                $message = "Failed to apply the discount to the charge chargeID: ".$subscriptionchargeID;
                Log::channel('subscriptionControllerLog')->info("otpToSubscription: ChargeId => ".$subscriptionchargeID.", 
                Error => ".$error);
                $this->requestResponse("Failed", $message, $error, $data->input('callback'));
            }
            $message = "Successfully created the subscription and applied the discount";
            $error = "";
            $status = "Success";
            $this->requestResponse($status, $message, $error, $data->input('callback'));
        }
        $message = "Successfully created the subscription";
        $error = "";
        $status = "Success";
        $this->requestResponse($status, $message, $error, $data->input('callback'));
    }
    
    public function requestResponse($status, $message, $error, $callback) {

        $response = array(
            "status"       => $status,
            "message" => $message,
            "error" => $error
        );
        echo $callback . '(' . json_encode($response) . ')';
        die();

    }
}
