<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\IterableApi;
use App\Jobs\ShopifyCustomer;
use Log;

class SurveyController extends Controller
{
    public $iterableObj;
    private $logDirectory;
    private $logFiles;
    private $logPath;

    public function __construct() {

        $this->iterableObj = new IterableApi();
    }

   /**
    * Update menopause quiz info to iterable
    */
    public function updateCustomer(Request $request) {
       
        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status" => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('surveyControllerLog')->info("updateCustomer: Endpoint parameters are missing");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
 
        Log::channel('surveyControllerLog')->info("updateCustomer: Request => ".$requestData);
        $requestData = json_decode($requestData);
        $email = !empty($requestData->email) ? trim($requestData->email) : '';
        $properties = !empty($requestData->properties) ? $requestData->properties : '';
 
        if (empty($email)) {
            $response = array(
                "status" => "Failed",
                "errorMessage" => "Missing required email",
            );
 
            Log::channel('surveyControllerLog')->info("updateCustomer: Missing required email");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
 
        if (empty($properties)) {
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required properties",
            );
 
            Log::channel('surveyControllerLog')->info("updateCustomer: Missing required properties");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
 
        $assessment = '';
        $acceptsMarketing = false;
        $customProperties = [];
        
        foreach ($properties as $key => $value) {
            
            $customProperties[$key] = $value;
            if ($key == 'assessment') {
                $assessment = $value;
            }
            if ($key == 'acceptsMarketing' && trim(strtolower($value)) == 'yes') {
                $acceptsMarketing = true;
            }
            if (trim($key) == 'ageAtSignup' && !empty($value)) {
                $customProperties["userAge"] = (int)$value;
            }
        }
        $userIterableUpdateRequest = array(
            "email" => $email,
            "dataFields" => array(
                "quizAnswer" => $customProperties,
                "assessment" => $assessment,
                "accepts_marketing" => $acceptsMarketing
            )
        );

        $eventRequest = array(
            "email" => strtolower($email),
            "eventName" => "quizCompletion"
        );
    
        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
            Log::channel('surveyControllerLog')->info("updateCustomer: Quiz Completion event - Successfully created the event, Email: ".
            strtolower($email));
        } else {
            Log::channel('surveyControllerLog')->info("Quiz Completion event - Failed to create the event, Email: ".
            strtolower($email));
            Log::channel('surveyControllerLog')->info("updateCustomer: Response =>  ".json_encode($eventResponse));
        }
        
        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));
 
        if (strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
            
            Log::channel('surveyControllerLog')->info("updateCustomer: Successfully updated the customer properties to iterable, Email - ".$email);
 
            $response = array(
                "status" => "success",
                "message" => "Successfully updated the customer properties - ".$email,
                "errorMessage" => "",
            );
 
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
           
        } else {
            Log::channel('surveyControllerLog')->info("updateCustomer: Failed to update the customer properties - ".$email);
            Log::channel('surveyControllerLog')->info("updateCustomer: Response =>  ".json_encode($userIterableUpdateResponse));
 
            $response = array(
                "status" => "failed",
                "message" => "Failed to update the customer properties - ".$email,
                "errorMessage"  => json_encode($userIterableUpdateResponse),
            );
    
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
    }
    
    
    /**
     * Create quiz completion event
     */
    public function quizCompletionEvent(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status" => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('surveyControllerLog')->info("quizCompletionEvent: Endpoint parameters are missing");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
        Log::channel('surveyControllerLog')->info("quizCompletionEvent: Request => ".$requestData);
        $requestData = json_decode($requestData);

        $email = !empty($requestData->email) ? trim($requestData->email) : '';
        
        if (empty($email)) {
            $response = array(
                "status" => "Failed",
                "errorMessage" => "Missing required email",
            );
            Log::channel('surveyControllerLog')->info("quizCompletionEvent: Missing required email");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }

        $eventRequest = array(
            "email" => $email,
            "eventName" => "quizCompletion"
        );

        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
            Log::channel('surveyControllerLog')->info("quizCompletionEvent: quiz completion event - Successfully created 
            event for Email(".strtolower($email).")");

            $response = array(
                "status" => "success",
                "message" => "quiz completion event - Successfully created event  - ".$email,
                "errorMessage" => "",
            );
    
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();

        } else {
            Log::channel('surveyControllerLog')->info("quizCompletionEvent: quiz completion event - Failed to create the event 
            for Email (".strtolower($email).")");
            Log::channel('surveyControllerLog')->info("quizCompletionEvent: Response =>  ".json_encode($eventResponse));

            $response         = array(
                "status" => "failed",
                "message" => "quiz completion event - Failed to create the event - ".$email,
                "errorMessage"  => json_encode($eventResponse),
            );
    
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
    }

    /**
     * Create prospect event
     */
    public function sendProspectEvent(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('surveyControllerLog')->info("sendProspectEvent: Endpoint parameters are missing");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }

        Log::channel('surveyControllerLog')->info("quizCompletionEvent: Request => ".$requestData);
        $requestData = json_decode($requestData);

        $email = !empty($requestData->email) ? trim($requestData->email) : '';
        
        if (empty($email)) {
            $response = array(
                "status" => "Failed",
                "errorMessage" => "Missing required email",
            );
            Log::channel('surveyControllerLog')->info("sendProspectEvent: Missing required email");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
        
        $userIterableUpdateRequest = array(
            "email" => $email,
            "dataFields" => array(
                "accepts_marketing" => true
            )
        );
        
        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));
        
        if (isset($userIterableUpdateResponse->code) && strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
            Log::channel('surveyControllerLog')->info("sendProspectEvent: Successfully updated the accepted marketing properties - ".$email);
        }else{
            Log::channel('surveyControllerLog')->info("sendProspectEvent: Failed to update the accept marketing - ".$email);
            Log::channel('surveyControllerLog')->info("sendProspectEvent: Response => ".json_encode($userIterableUpdateResponse));
        }

        $eventRequest = array(
            "email" => $email,
            "eventName" => "prospect"
        );

        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
            Log::channel('surveyControllerLog')->info("sendProspectEvent: Prospect event - Successfully created event for Email (".
            strtolower($email).")");

            $response = array(
                "status" => "success",
                "message" => "Prospect event - Successfully created event  - ".$email,
                "errorMessage" => "",
            );
    
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();

        } else {
            Log::channel('surveyControllerLog')->info("sendProspectEvent: Prospect event - Failed to create the event for Email (".
            strtolower($email).")");
            Log::channel('surveyControllerLog')->info("sendProspectEvent: Response => ".json_encode($eventResponse));

            $response = array(
                "status" => "failed",
                "message" => "Prospect event - Failed to create event - ".$email,
                "errorMessage" => json_encode($eventResponse),
            );
    
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
    }
    
    /**
     * Create prospect notification event
     */
    public function sendProspectNotification(Request $request) {

        $requestData = !empty($request->input('data')) ? $request->input('data') : "";
        if (empty($requestData)) {
            $message = "Endpoint parameters are missing";
            $error = "Bad request";
            $status = "Failed";
            $response = array(
                "status"       => $status,
                "message" => $message,
                "error" => $error
            );
            Log::channel('surveyControllerLog')->info("sendProspectNotification: Endpoint parameters are missing");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }

        Log::channel('surveyControllerLog')->info("sendProspectNotification: Request => ".$requestData);
        $requestData = json_decode($requestData);

        $email = !empty($requestData->email) ? trim($requestData->email) : '';
        
        if (empty($email)) {
            $response = array(
                "status"       => "Failed",
                "errorMessage" => "Missing required email",
            );
            Log::channel('surveyControllerLog')->info("sendProspectNotification: Missing required email");
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
        
        Log::channel('surveyControllerLog')->info("sendProspectNotification: dispatched to create the customer in Shopify - ".$email);
                        
    	$shCustomerCreateResponse = dispatch(new ShopifyCustomer($requestData));
        
        $eventRequest = array(
            "email" => $email,
            "eventName" => "prospect"
        );

        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
            Log::channel('surveyControllerLog')->info("sendProspectNotification: Prospect event - Successfully created event for Email (".
            strtolower($email).")");

            $response = array(
                "status" => "success",
                "message" => "Prospect event - Successfully created event  - ".$email,
                "errorMessage" => "",
            );
    
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();

        } else {
            Log::channel('surveyControllerLog')->info("sendProspectNotification: Prospect event - Failed to create the event for Email (".
            strtolower($email).")");
            Log::channel('surveyControllerLog')->info("sendProspectNotification: Response => ".json_encode($eventResponse));

            $response         = array(
                "status" => "failed",
                "message" => "Prospect event - Failed to create the event - ".$email,
                "errorMessage"  => json_encode($eventResponse),
            );
    
            echo $request->input('callback') . '(' . json_encode($response) . ')';
            die();
        }
    }
}
