<?php
namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Models\RechargeApi;
use App\Models\IterableApi;
use Log;

class IterableOnetimeCreateEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $iterableObj;
    private $rechargeObj;
    private $customerID;
    private $properties;
    private $requestData;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($requestData) {

        $this->requestData = $requestData;
        $this->customerID  = $this->requestData->order->customer_id;
    }

    public function handle() {

        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($this->customerID);
        Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: One time order events create job");

        if (isset($customerDetails->error) || isset($customerDetails->errors) || empty($customerDetails->customer->email) ||
        empty($customerDetails)) {

            $status = "failed";
            $error = "Unknown error";
            if (isset($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } else if (isset($customerDetails->errors)) {
                $error = json_encode($customerDetails->error);
            } else {
                $error = "The request is not correct or missing the required parameters";
            }

            Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: Failed to get the Recharge user details for CustomerID (".
            $this->customerID."), Error: " . $error);
            die();
        }

        $eventRequest = array(
            "email" => $customerDetails->customer->email, 
            "eventName" => "singleBottlePurchase",
            "dataFields" => array(
                "userName" => $customerDetails->customer->first_name.' '.$customerDetails->customer->last_name
            )
        );

        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {

            Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: Created OTP iterable event - Successfully created event for Email (".
            strtolower($customerDetails->customer->email).")");
        } else {
            Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: Created OTP iterable event - Failed to create the event for Email (".
            strtolower($customerDetails->customer->email).")");
            Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: Response => ".json_encode($eventResponse));
        }
        
        $userIterableUpdateRequest = array(
            "email" => $customerDetails->customer->email,
            "dataFields" => array(
                "rechargeHash" => $customerDetails->customer->hash,
                "lastOtpOrderId" => $this->requestData->order->id
            )
        );

        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));

        if (strtolower(trim($userIterableUpdateResponse->code)) == 'success') {

            Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: User rechargeHash - Successfully updated the user 
            recharge Hash in Iterable for Email (".strtolower($customerDetails->customer->email).")");
        } else {
            Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: User rechargeHash - Failed to update the user recharge Hash 
            in Iterable for Email (".strtolower($customerDetails->customer->email).")");
            Log::channel('iterableWebhookLog')->info("orderCreateEventWebhook: Response => ".json_encode($userIterableUpdateResponse));
        }
        
    }

}
