<?php

namespace App\Jobs;

use App\Models\IterableApi;
use App\Models\RechargeApi;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Log;

class IterableSubscriptionCancelEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $iterableObj;
    private $rechargeObj;
    private $customerID;
    private $properties;
    private $requestData;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($requestData) {

        $this->requestData = $requestData;
        $this->customerID = $requestData;
    }

    public function handle() {

        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($this->customerID);
        
        Log::channel('iterableWebhookLog')->info("IterableSubscriptionCancelEvent: Subscription cancelled event create job");
        Log::channel('iterableWebhookLog')->info("IterableSubscriptionCancelEvent: customerDetails".json_encode($customerDetails));

        if (isset($customerDetails->error) || isset($customerDetails->errors) ||
            empty($customerDetails->customer->email) || empty($customerDetails)) {
            $status = "failed";
            $error = "Unknown error";

            if (isset($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } elseif (isset($customerDetails->errors)) {
                $error = json_encode($customerDetails->errors);
            } else {
                $error = "The request is not correct or missing the required parameters";
            }
                
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCancelEvent: Cancelled subscription iterable event - Failed to get 
            the Recharge user details for CustomerID (" .$this->customerID . ")");
            die();
        }

        $eventRequest = array(
            "email" => strtolower($customerDetails->customer->email),
            "eventName" => "subscriptionCancelled",
            "dataFields" => array(
                "userName" => $customerDetails->customer->first_name . ' ' . $customerDetails->customer->last_name,
            )
        );

        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));
        
        Log::channel('iterableWebhookLog')->info("IterableSubscriptionCancelEvent: eventResponse".json_encode($eventResponse));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
           
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCancelEvent: Cancelled subscription iterable event - Successfully created 
            event for Email (" .strtolower($customerDetails->customer->email) . ")");
        } else {
            
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCancelEvent: Cancelled subscription iterable event - Failed to create 
            event for Email (" .strtolower($customerDetails->customer->email) . ")");

            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCancelEvent: Response => ".json_encode($eventResponse));
        }
    }
}
