<?php

namespace App\Jobs;

use App\Models\IterableApi;
use App\Models\RechargeApi;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Log;

class IterableSubscriptionChargeUpcomingEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $iterableObj;
    private $rechargeObj;
    private $webhookResponse;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($webhookResponse) {

        $this->webhookResponse = $webhookResponse;
    }

    public function handle() {

        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();

        $nextChargeDate = '';
        $subscriptionLineItemsArray = [];
        $subscriptionLineItems = '';
        
        Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Subscription charge upcoming event job");
        
        if (!empty($this->webhookResponse['charge']['line_items'])) {
            $nextChargeDate = !empty($this->webhookResponse['charge']['scheduled_at']) ?
            trim($this->webhookResponse['charge']['scheduled_at']) : '';

            foreach ($this->webhookResponse['charge']['line_items'] as $chargeValue) {
                $subscriptionLineItemsArray[] = $chargeValue['title'];
            }
        }

        if (!isset($subscriptionLineItemsArray) || empty($subscriptionLineItemsArray) ||
            empty($this->webhookResponse['charge']['email'])) {
            if (empty($this->webhookResponse['charge']['email'])) {
                $error = "Upcoming Charge iterable event - The email address is not valid or the field is empty";
            } else {
                $error = "Upcoming Charge iterable event - The charge line items are not valid or the line
                items are empty(Email - " . $this->webhookResponse['charge']['email'] . ")";
            }
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Error => ".$error);
            die();
        }
        
        $firstName = '';
        if (isset($this->webhookResponse['charge']['first_name']) && !empty($this->webhookResponse['charge']['first_name'])) {
            $firstName = $this->webhookResponse['charge']['first_name'];
        }
        
        $discountCode = '';
        if (isset($this->webhookResponse['charge']['discount_codes']) && !empty($this->webhookResponse['charge']['discount_codes'])) {
            $discount = $this->webhookResponse['charge']['discount_codes'][0];
            $discountCode = $discount['code'];
        }
        
        $totalPrice = '';
        if (isset($this->webhookResponse['charge']['total_price']) && !empty($this->webhookResponse['charge']['total_price'])) {
            $totalPrice = $this->webhookResponse['charge']['total_price'];
        }

        if (!empty($nextChargeDate)) {

            $nextChargeDate = strtotime($nextChargeDate);
            $nextChargeDate = date("Y-m-d H:i:s", $nextChargeDate);
        }
        $subscriptionLineItems = implode(' | ', $subscriptionLineItemsArray);

        $eventRequest = array(
            "email" => strtolower($this->webhookResponse['charge']['email']), 
            "eventName" => "upcomingCharge",
            "dataFields" => array(
                "nextChargeDate" => $nextChargeDate, 
                "subscriptionItems" => $subscriptionLineItems,
                "totalPrice" => $totalPrice,
                "firstName" => $firstName,
                "couponCode" => $discountCode
            )
        );

        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {

            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Upcoming Charge iterable event - Successfully 
            created event for Email (" .strtolower($this->webhookResponse['charge']['email']) . ")");
            
        } else {
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Upcoming Charge iterable event - Failed to 
            create the event for Email (" .strtolower($this->webhookResponse['charge']['email']) . ")");

            Log::channel('iterableWebhookLog')->info("IterableSubscriptionChargeUpcomingEvent: Response => ".json_encode($eventResponse));
        }
    }
}
