<?php

namespace App\Jobs;

use App\Models\IterableApi;
use App\Models\RechargeApi;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Log;

class IterableSubscriptionCreateEvent implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $iterableObj;
    private $rechargeObj;
    private $customerID;
    private $properties;
    private $requestData;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($requestData) {

        $this->requestData = $requestData;
        $this->customerID = $this->requestData->subscription->customer_id;
        $this->properties = $this->requestData->subscription->properties;
    }

    public function handle() {

        $this->iterableObj = new IterableApi();
        $this->rechargeObj = new RechargeApi();

        Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: Subscription created event job");

        $customerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($this->customerID);

        Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: customerDetails => ".json_encode($customerDetails));
        
        if (isset($customerDetails->error) || isset($customerDetails->errors) ||
            empty($customerDetails->customer->email) || empty($customerDetails)) {
            $status = "failed";
            $error = "Unknown error";

            if (isset($customerDetails->error)) {
                $error = json_encode($customerDetails->error);
            } elseif (isset($customerDetails->errors)) {
                $error = json_encode($customerDetails->errors);
            } else {
                $error = "The request is not correct or missing the required parameters";
            }
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: Created subscription iterable event - Failed to get 
            the Recharge user details for CustomerID (" .$this->customerID . ")");
            die();
        }

        $eventRequest = array(
            "email" => strtolower($customerDetails->customer->email), 
            "eventName" => "subscriptionCreated"
        );

        $userIterableUpdateRequest = array(
            "email" => $customerDetails->customer->email,
            "dataFields" => array(
                "rechargeHash" => $customerDetails->customer->hash,
            ),
        );

        $userIterableUpdateResponse = $this->iterableObj->updateUserDataFields(json_encode($userIterableUpdateRequest));

        if (strtolower(trim($userIterableUpdateResponse->code)) == 'success') {
            
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: User rechargeHash and subscription properties - Successfully
            updated the user recharge Hash and properties in Iterable for Email (" .strtolower($customerDetails->customer->email) . ")");
            
        } else {
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: User rechargeHash and subscription properties - Failed to update
            the user recharge Hash and properties in Iterable for Email (" .strtolower($customerDetails->customer->email) . ")");
        }
      
        $eventResponse = $this->iterableObj->createIterableEvent(json_encode($eventRequest));
        
        if (isset($eventResponse->code) || trim(strtolower($eventResponse->code)) == 'success') {
            
            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: Created subscription iterable event - Successfully created 
            event for Email (" .strtolower($customerDetails->customer->email) . ")");
            
        } else {

            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: Created subscription iterable event - Failed to create the 
            event for Email (" .strtolower($customerDetails->customer->email) . ")");

            Log::channel('iterableWebhookLog')->info("IterableSubscriptionCreateEvent: Response => ".json_encode($eventResponse));
        }
    }
}
