<?php
namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\Models\RechargeApi;
use App\Models\IterableApi;
use App\Models\Shopify\Customer;
use Osiset\BasicShopifyAPI\BasicShopifyAPI;
use Osiset\BasicShopifyAPI\Options;
use Osiset\BasicShopifyAPI\Session;
use Log;

class ShopifyCustomer implements ShouldQueue
{
    use InteractsWithQueue, Queueable;

    private $requestData;
    private $shopify_api_version;
    private $api;
    private $customerObj;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($requestData) {

        $this->requestData = $requestData;
    }

    public function handle() {

        $this->customerObj = new Customer();

        // Create options for the API
        $options = new Options();
        $options->setVersion(env('SHOPIFY_API_VERSION', true));

        // Create the client and session
        $this->api = new BasicShopifyAPI($options);
        $this->api->setSession(new Session(env('SHOPIFY_DOMAIN', true), env('SHOPIFY_API_PASSWORD', true)));
        
        $this->shopify_api_version = env('SHOPIFY_API_VERSION');
        
        Log::channel('surveyControllerLog')->info("ShopifyCustomer: Customer create job");
        
        if (isset($this->requestData->email)) {
            
            $customerAvailable = false;


            $customerDetails = $this->customerObj->getCustomerDetailBySearch($this->requestData->email);

            if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

                $error = "Unknown error";
                if (!empty($customerDetails['error'])) {

                    $error = json_encode($customerDetails['error']);
                } elseif (!empty($customerDetails['errors'])) {
                    
                    $error = json_encode($customerDetails['errors']);
                }
                if (empty($error)) {

                $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $$this->requestData->email . ")";
                }
                $returnData = [
                    "error" => $error
                ];

                Log::channel('surveyControllerLog')->info("ShopifyCustomer: getCustomerActivationStatus: Failed to get the 
                customer details from Shopify(emailAddress:".$this->requestData->email."), Error: (".$error.")");
                //$this->sendResponse(600, "error", $returnData);
            }

            if (!isset($customerDetails['customers'][0]) || empty($customerDetails['customers'][0])) {

                Log::channel('surveyControllerLog')->info("ShopifyCustomer: No such customer was found in Shopify (search) -".$this->requestData->email);
            } else {

                foreach ($customerDetails['customers'] as $cust) {
                    if (trim(strtolower($cust['email'])) == trim(strtolower($this->requestData->email))) {
                        $customerAvailable = true;
                    }
                }
            }
            
            if (!$customerAvailable) {

                $params = array(
                    'customer' => array(
                        'email' => $this->requestData->email,
                        //'accepts_marketing' => 'true'
                        "email_marketing_consent" => array(
                            "state" => "subscribed",
                            "opt_in_level" => "single_opt_in"
                        )
                    )
                );
                $retry = false;
                $try = 1;
                do {
                    Log::channel('surveyControllerLog')->info("ShopifyCustomer:".$try.") Trying to create the customer in Shopify - ".$this->requestData->email);
                    
                    $result = $this->api->rest('POST', '/admin/api/'.$this->shopify_api_version.'/customers.json', $params);
                    if (empty($result->body->customer->id)) {
                        $retry = true;
                        $try++;
                    }
        
                } while($retry && $try <=3);
                
                if (!empty($result->body->customer->id)) {
                    Log::channel('surveyControllerLog')->info("ShopifyCustomer: Successfully created customer, Email -".$this->requestData->email);
                    
                } else {
                    Log::channel('surveyControllerLog')->info("ShopifyCustomer: Failed to create customer, Email -".$this->requestData->email);
                    Log::channel('surveyControllerLog')->info("ShopifyCustomer: Response => ".json_encode($result));
                }
            } else {
                Log::channel('surveyControllerLog')->info("ShopifyCustomer: The customer already exists in Shopify, Email -".$this->requestData->email);
            }           

        }

    }

}
